unit UMLAddinImp;

interface
  uses ComObj, SysUtils, OleCtrls, Dialogs, UMLAddin_TLB, EA_TLB;

  type
    TUMLAddin = class(TAutoObject, IUMLAddin)
    protected
      function IUMLAddin.EA_Connect = EA_Connect;
      function EA_Connect(const Repos: IDispatch): WideString; safecall;

      procedure IUMLAddin.EA_Disconnect = EA_Disconnect;
      procedure EA_Disconnect; safecall;

      function IUMLAddin.EA_GetMenuItems = EA_GetMenuItems;
      function EA_GetMenuItems(const Repos: IDispatch; const MenuLocation: WideString;
                             const MenuName: WideString): OleVariant; safecall;

      procedure IUMLAddin.EA_MenuClick = EA_MenuClick;
      procedure EA_MenuClick(const Repos: IDispatch; const MenuLocation: WideString;
                           const MenuName: WideString; const ItemName: WideString); safecall;

      procedure IUMLAddin.EA_GetMenuState = EA_GetMenuState;
      procedure EA_GetMenuState(const Repos: IDispatch; const MenuLocation: WideString;
                              const MenuName: WideString; const ItemName: WideString; 
                              var IsEnabled: WordBool; var IsChecked: WordBool); safecall;

      procedure Process_Method(m: Method; sync: Boolean);

      function UMLFormatParameter(param: Parameter): WideString;
    end;

implementation
  uses ComServ, Variants;

  const
    MENU_DIAGRAM          = 'Diagram';
    MENU_TREEVIEW         = 'TreeView';
    MENU_MAIN             = '-&UMLAddin';
    MENU_ITEM_SYNCHRONIZE = '&Update Parameters Alias';
    MENU_ITEM_CLEAR_ALIAS = '&Clear Parameters Alias';

  function TUMLAddin.EA_Connect(const Repos: IDispatch): WideString;
  begin
    {ShowMessage('EA_Connect');}
    Result := '';
  end;

  procedure TUMLAddin.EA_Disconnect;
  begin
    {ShowMessage('EA_Disconnect');}
  end;

  function TUMLAddin.EA_GetMenuItems(const Repos: IDispatch; const MenuLocation: WideString;
                             const MenuName: WideString): OleVariant;
  var
    menu  : Variant;
  begin
    if (MenuLocation = MENU_DIAGRAM) or (MenuLocation = MENU_TREEVIEW) then begin
      if (MenuName = '') then begin
        Result := MENU_MAIN;
      end else begin
        if (MenuName = MENU_MAIN) then begin
          menu    := VarArrayCreate([0,1],varOleStr);
          menu[0] := MENU_ITEM_SYNCHRONIZE;
          menu[1] := MENU_ITEM_CLEAR_ALIAS;
          Result  := menu;
        end;
      end;
    end;
  end;

  procedure TUMLAddin.EA_GetMenuState(const Repos: IDispatch; const MenuLocation: WideString;
                              const MenuName: WideString; const ItemName: WideString;
                              var IsEnabled: WordBool; var IsChecked: WordBool);
  var
    rep     : Repository;
    obj     : IDispatch;
    objType : ObjectType;
  begin
    IsChecked := False;
    IsEnabled := False;
    //if (MenuLocation = MENU_DIAGRAM) or (MenuLocation = MENU_TREEVIEW) then begin
      if (MenuName = MENU_MAIN) then begin
        rep := Repos as Repository;
        objType := rep.GetContextItem(obj);
        if (objType = otMethod) or (
          (objType = otElement) and ( ( (obj as Element).Type_ = 'Class')  or ( (obj as Element).Type_ = 'Interface'))
        ) then begin
          IsEnabled := True;
        end;
      end;
    //end;

  end;

  procedure TUMLAddin.EA_MenuClick(const Repos: IDispatch; const MenuLocation: WideString;
                           const MenuName: WideString; const ItemName: WideString);
  var
    i       : Integer;
    rep     : Repository;
    obj     : IDispatch;
    objType : ObjectType;
    e       : Element;
    m       : Method;
    d       : Diagram;

  begin
    if (MenuName = MENU_MAIN) then begin
      rep := Repos as Repository;

      //      .
      objType := otNone;
      if (MenuLocation = MENU_DIAGRAM) then begin
        objType := rep.GetContextItem(obj);
      end else if (MenuLocation = MENU_TREEVIEW) then begin
        objType := rep.GetTreeSelectedItem(obj);
      end;

      //    alias-
      //   
      if (objType = otMethod) then begin
        m := obj as Method;
        self.Process_Method(m, ItemName = MENU_ITEM_SYNCHRONIZE);

        e := rep.GetElementByID(m.ParentID);
        if not e.Update() then
          ShowMessage(e.GetLastError());
      end;

      //    (   !)
      if (objType = otElement) then begin
        e := obj as Element;
        if (e.Type_ = 'Class') or (e.Type_ = 'Interface') then begin
          for i := 0 to e.Methods.Count - 1 do begin
            self.Process_Method(
              e.Methods.GetAt(i) as Method,
              ItemName = MENU_ITEM_SYNCHRONIZE
              );
          end;
          if not e.Update() then
            ShowMessage(e.GetLastError());
        end;
      end;

      //      .
      // :TODO:       ?
      d := rep.GetCurrentDiagram();
      if (d <> nil) and (not d.Update()) then
        ShowMessage(d.GetLastError());
    end;

  end;

  procedure TUMLAddin.Process_Method(m: Method; sync: Boolean);
  var
    i : Integer;
    p : Parameter;
  begin
    for i := 0 to m.Parameters.Count - 1 do begin
      p := m.Parameters.GetAt(i) as Parameter;

      if sync then begin
        p.Alias := self.UMLFormatParameter(p);
      end else begin
        p.Alias := '';
      end;

      if not p.Update() then
        ShowMessage(p.GetLastError());
    end;
    if not m.Update() then
      ShowMessage(p.GetLastError());
  end;

  function TUMLAddin.UMLFormatParameter(Param: Parameter): WideString;
  var
    str: WideString;
  begin
    str:= '';
    if Param.Kind <> 'in' then begin
      str:= str + '<' + Param.Kind + '> ';
    end;

    str:= str + Param.Name;

    if Param.Type_ <> '' then begin
      str:= str + ': ' + Param.Type_;
    end;

    if Param.Default <> '' then begin
      str := str + ' = ' + Param.Default;
    end;

    Result := str;
  end;

initialization
  TAutoObjectFactory.Create(ComServer, TUMLAddin, CLASS_Addin, ciMultiInstance);
end.
